#include "yacc2html.h"

extern char * title, * input_file;
extern char * program_name, * fmt_terminal_url;
extern char * yacc_sed_file, * lex_sed_file, * yacc_file;
extern Hashtable nonterminals, terminals;
extern char * nonterminal_name(char const *);

write_yacc_file()
{
	FILE * f;
	char const * n;
	char const * p;

	if (!(f = fopen(yacc_sed_file, "w"))) {
		perror(yacc_sed_file);
		return -1;
	}

	for (n = hashnext(nonterminals, (char*)0);
						n; n=hashnext(nonterminals, n)){
		p = hashname(nonterminals, n);
		assert(p);
		if (!definedP(p))
			fprintf(f, "s\b\\<%s\\>\b<a href=\"%s#%s\">%s</a>\bg\n",
			p, yacc_file, nonterminal_name(p), p);
	}
	if (ferror(f) || fclose(f)) {
		fprintf(stderr,
		"%s: panic: error writing yacc sed-script \"%s\"\n",
			program_name,
			yacc_sed_file);
		return -1;
	}
	return 0;
}

static char const * terminal_fmt()
{
	static char const * fmt_terminal_name;
	char const  * c;

	if (!fmt_terminal_name) {
		c = strchr(fmt_terminal_url, '#');
		if (c) fmt_terminal_name = c + 1;
		else fmt_terminal_name = "%s";
	}
	return fmt_terminal_name;
}

write_lex_file()
{
	FILE * f;
	char const * n, * p;

	if (!(f = fopen(lex_sed_file, "w"))) {
		perror(lex_sed_file);
		return -1;
	}

	fprintf(f, "1i\\\n<html><head><title>%s (Lex)\\\n</title><body><pre>\n",
		title ? title : input_file);

	fprintf(f, "1,$s/\\&/\\&amp;/g\n");
	fprintf(f, "1,$s/\\\"/\\&quot;/g\n");
	fprintf(f, "1,$s/</\\&lt;/g\n");
	fprintf(f, "1,$s/>/\\&gt;/g\n");

	for (n = hashnext(terminals, (char*)0); n; n = hashnext(terminals, n)){
		p = hashname(terminals, n);
		assert(p);
		if (!definedP(p))
			fprintf(f,
			"1,/\\<%s\\>/s\b\\<%s\\>\b<a name=\"%s\">&</a>\b\n",
			p, p, uprint(terminal_fmt(), p));
	}
	fprintf(f, "$a\\\n</pre></body></html>\n");
 
	if (ferror(f) || fclose(f)) {
		fprintf(stderr,
		"%s: panic: error writing lex sed-script \"%s\"\n",
			program_name,
			lex_sed_file);
		return -1;
	}
	return 0;
}
